import logging
from google.protobuf import json_format

from .core.text_annotation import TextAnnotation
from .protobuf import TextAnnotation_pb2
from . import utils

logging.basicConfig(level=logging.INFO)

def load_document_from_json(file_name):
    """
    Load a TextAnnotation document from a JSON file.

    Returns a TextAnnotation instance.
    """

    json_data = None
    with open(file_name, 'rb') as f:
        json_data = f.read()

    return TextAnnotation(json_data)


def load_document_from_protobuf(file_name):
    """
    Load a TextAnnotation document from a protocol buffer encoded file.

    Returns a TextAnnotation instance.
    """

    proto_data = None
    with open(file_name, 'rb') as f:
        proto_data = f.read()

    message = TextAnnotation_pb2.TextAnnotationProto()
    message.ParseFromString(proto_data)

    # Currently convering the protobuf object to a JSON string and pass it to TextAnnotation
    # TODO - Parse protobuf directly instead of JSON when TextAnnotation is more mature.
    return TextAnnotation(json_format.MessageToJson(message))



import json
import logging

logger = logging.getLogger(__name__)

class View(object):
    def __str__(self):
        """
        Special method to print the view in (label, tokens) format
        """
        constituent_label_string = ""
        if self.cons_list is None:
            constituent_label_string = "this view does not have constituents in your input text. "
        else:
            for cons in self.cons_list:
                constituent_label_string += "(" + cons['label'] + " " + cons['tokens'] + ") "
        return self.view_name + " view: " + constituent_label_string

    # The three following functions are used to make View class be able to iterate and index
    def __iter__(self):
        index = 0
        while index < len(self.cons_list):
            yield self.cons_list[index]
            index += 1

    def __getitem__(self, index):
        return self.cons_list[index]

    def __len__(self):
        return len(self.cons_list)


    def __init__(self, view, tokens):
        """
        Constructor for the view

        @param: view, the decoded JSON object containing information of the view
                tokens, List of tokens in the view
        """
        self.view_name = view["viewName"]
        self.tokens = tokens

        # get view_type: TreeView, PredicateArgument, TokenLabelView, ...
        full_type = view["viewData"][0]["viewType"]
        self.generator = view["viewData"][0]["generator"]
        split_by_period = full_type.split(".")
        # self.view_type = split_by_period[len(split_by_period) - 1]
        self.view_type = full_type

        self.cons_list = None
        self.relation_array = None
        self.labels_to_scores_array = None

        if "constituents" in view["viewData"][0] and view["viewData"][0]["constituents"] != None:
            self.cons_list = []
            for constituent in view["viewData"][0]["constituents"]:
                # Labels of TOKENS view will not be recorded when serializing text annotation in JSON format in pipeline
                # So there is a statement for handling this 
                cons_tokens = self.tokens[constituent['start']]
                for index in range(constituent['start']+1, constituent['end']):
                    cons_tokens += ' '
                    cons_tokens += self.tokens[index]
                if self.view_name == 'TOKENS':
                    constituent['label'] = cons_tokens
                constituent['tokens'] = cons_tokens
                self.cons_list.append(constituent)

        if "relations" in view["viewData"][0]:
            self.relation_array = []
            for relation in view["viewData"][0]["relations"]:
                self.relation_array.append(relation)
            self._link_constituents()

        if "labelsToScores" in view["viewData"][0] and view["viewData"][0]["labelsToScores"] != None:
            self.labels_to_scores_array = []
            for l2s_map in view["viewData"][0]["labelsToScores"]:
                self.labels_to_scores_array.append(l2s_map)

    def _link_constituents(self):
        """
        Helper function to build connection between constituents based on relation
        This function will be called only when relations exist
        """
        for relation_index in range(len(self.relation_array)):
            relation = self.relation_array[relation_index]
            src = self.cons_list[relation['srcConstituent']]
            target = self.cons_list[relation['targetConstituent']]
            if 'outgoing_relations' not in src:
                src['outgoing_relations'] = []
            if 'incoming_relations' not in target:
                target['incoming_relations'] = []
            src['outgoing_relations'].append(relation_index)
            target['incoming_relations'].append(relation_index)

    def get_view_type(self):
        """
        Function to get type of the view

        @return view type of the view
        """
        return self.view_type

    def get_cons(self, position=None, key=None):
        """
        Function to get a list of constituents in the view

        @param: position, the index of the specific constituent that user wants
                key, the specific key in constituents that user wants ("score", "label", "position","tokens")
        @return: if key is not given, a list of all constituents if position is not given,
                 or a list contains the constituent at specified position if position is given
                 otherwise, a list of specific key in respect to constituents

        """
        if self.cons_list is None:
            logger.warn("This view does not have constituents in your input text")
            return None

        if key is None:
            if position is not None and 0 <= position < len(self.cons_list):
                return [self.cons_list[position]]
            else:
                return self.cons_list
        elif key == "score" or key == "label" or key == "position" or key == "tokens":
            result_list = []
            if position is not None and 0 <= position < len(
                    self.cons_list):
                if key == "position":
                    result_list.append((self.cons_list[position]["start"],
                                        self.cons_list[position]["end"]))
                else:
                    result_list.append(self.cons_list[position][key])
            else:
                for constituent in self.cons_list:
                    if key == "position":
                        result_list.append(
                            (constituent["start"], constituent["end"]))
                    else:
                        result_list.append(constituent[key])
            return result_list

        logger.warn("Invalid key in constituent")
        return None

    def get_con_score(self, position=None):
        """
        Wrapper function to get a list of scores of constituents in the view

        @param: position, the index of the specific constituent that user wants score from
        @return: list of scores of all constituents if position is not given,
                 otherwise return a list contains the score of the constituent at specified position

        """
        return self.get_cons(position, "score")

    def get_con_label(self, position=None):
        """
        Wrapper function to get a list of labels of constituents in the view 

        @param: position, the index of the specific constituent that user wants label from 
        @return: list of labels of all constituents if position is not given, 
                 otherwise return a list contains the label of the constituent at specified position 
        """
        return self.get_cons(position, "label")

    def get_labels_to_scores(self, position=None):
        """
        Function to get the labelsToScores list of maps if the view supports it.

        @param: position, index of specific labelsToScores map the users wants
        @return: a list of all labelsToScores maps for all constituents if position is not specified
                 otherwise returns a singletion list with the specific map
        """
        if self.labels_to_scores_array is None:
            logger.warn("This view does not support labelsToScores.")
            return None
        else:
            if position is not None and 0 <= position < len(
                    self.labels_to_scores_array):
                return [self.labels_to_scores_array[position]]
            else:
                return self.labels_to_scores_array
           
    def get_con_position(self, position=None):
        """
        Wrapper function to get a list of positions of constituents in the view in respect to tokens of the text

        @param: position, the index of the specific constituent that user wants token position from
        @return: list of position tuples (start_pos, end_pos) of all constituents if position is not given, 
                 otherwise return a list contains the token position of the constituent at specified position 
        """
        return self.get_cons(position, "position")

    def get_relations(self, position=None):
        """
        Funtion to get the relation array if the view supports relations

        @param: position, the index of the specific relation that user wants
        @return: list of relations if position is not given,
                 otherwise return a list contains the relation at specified position
        """
        if self.relation_array is None:
            logger.warn("This view does not support relations")
            return None
        else:
            if position is not None and 0 <= position < len(
                    self.relation_array):
                return [self.relation_array[position]]
            else:
                return self.relation_array

    def get_overlapping_constituents(self, start_token_index, end_token_index):
        """
        Function to get a list of constituents in the view that overlap with the indices provided

        @param: start_token_index, the starting index of the range for overlapping
                end_token_index, the ending index of the range for overlapping
        @return: List of overlapping constituents if the indice are valid, None otherwise
        """
        if start_token_index > end_token_index:
            logger.warn("Invalid token indices given, please provide proper indices.")
            return None
        view_overlapping_span = []
        for cons in self.cons_list:
            if((cons['start'] <= start_token_index and cons['end'] >= start_token_index) or
                    (cons['start'] <= end_token_index and cons['end'] >= end_token_index) or
                    (cons['start'] >= start_token_index and cons['end'] <= end_token_index) or
                    (cons['start'] <= start_token_index and cons['end'] >= end_token_index)):
                view_overlapping_span.append(cons)
        return view_overlapping_span

    @property
    def as_json(self):
        output = {
            "viewName": self.view_name,
            "viewData": [
                {
                    "viewType": self.view_type,
                    "viewName": self.view_name,
                    "generator": self.generator,
                    "score": 1,
                }
            ]
        }

        if self.cons_list is not None:
            output["viewData"][0]["constituents"] = self.cons_list
        if self.labels_to_scores_array is not None:
            output["viewData"][0]["labelsToScores"] = self.labels_to_scores_array

        return output



import codecs
import configparser
import logging
import os
import platform
import subprocess
import six

logger = logging.getLogger(__name__)

MODEL_FOLDER = "model_{}"
CONFIG_FILENAME = "config.cfg"
DEFAULT_CONFIG_ROOT_DIRECTORY = "~{0}.ccg_nlpy{0}".format(os.path.sep)
DEFAULT_CONFIG_VERSION = "4.0.12"
MAVEN_COMMAND = "mvn dependency:copy-dependencies -DoutputDirectory={} -Dsilent=True"
POM_TEMPLATE = """<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0"
         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xsi:schemaLocation="http://maven.apache.org/POM/4.0.0 http://maven.apache.org/xsd/maven-4.0.0.xsd">
    <modelVersion>4.0.0</modelVersion>

    <groupId>org.cogcomp</groupId>
    <artifactId>ccg_nlpy-pipeline-dependencies</artifactId>
    <version>1.0-SNAPSHOT</version>

    <repositories>
        <repository>
            <id>CogcompSoftware</id>
            <name>CogcompSoftware</name>
            <url>http://cogcomp.org/m2repo/</url>
        </repository>
    </repositories>

    <dependencies>
        <dependency>
            <groupId>edu.illinois.cs.cogcomp</groupId>
            <artifactId>illinois-nlp-pipeline</artifactId>
            <version>##VERSION##</version>
        </dependency>
    </dependencies>

</project>
"""


def _shell_argument():
    """Identify if the current platform is a Windows installation."""
    return 'windows' in platform.system().lower()


def _parse_default_config(root_directory, args=None):
    """Parse default configuration for model_download"""

    default_config_file = os.path.join(root_directory, CONFIG_FILENAME)
    package_config_file = os.path.dirname(os.path.realpath(__file__)) + '/config/pipeline.cfg'

    config = configparser.ConfigParser()
    if os.path.exists(default_config_file):
        with codecs.open(default_config_file, mode='r', encoding='utf-8') as f:
            config.read_string(f.read())
    else:
        with codecs.open(package_config_file,mode='r',encoding='utf-8') as f:
            config.read_string(f.read())

    if 'model_download' not in config:
        config['model_download'] = {}

    if args is not None and args.version is not None:
        version = args.version
    elif 'version' not in config['model_download']:
        version = DEFAULT_CONFIG_VERSION
    else:
        version = config['model_download']['version']

    config['model_download']['version'] = version

    return (config, default_config_file)


def _check_maven_installed():
    """Method to check if maven is installed and on the classpath"""

    try:
        output = subprocess.check_output(
            ["mvn", "--version"], shell=_shell_argument())
        logger.debug(output)
    except Exception:
        logger.error(
            'Maven installation not found!\n\
            Please install Apache Maven and add it to the classpath.',
            exc_info=True)
        raise


def _create_or_update_pom_file(file_path, version):
    """
    Create the pom.xml file needed for maven to download dependencies.
    """

    logger.info("Creating pom file")

    if six.PY2:
        # Python 2 - Write as a binary file and replace newline on windows
        # to universal newline.
        with open(file_path, 'wb') as file:
            pom_data = POM_TEMPLATE.replace("##VERSION##", version)
            pom_data = pom_data.replace('\r\n', '\n')
            file.write(pom_data)
    else:
        # Python 3 - use newline as \n always. Maven does not work otherwise.
        with open(file_path, "w", newline='\n') as file:
            pom_data = POM_TEMPLATE.replace("##VERSION##", version)
            file.write(pom_data)


def _download_jars(model_directory, config_directory, version):
    """
    Download Jars into the model_directory parameter

    Note: This method invokes maven internally to download dependencies.
    """

    if not os.path.exists(model_directory):
        os.makedirs(model_directory)

    logger.info('Root Directory = {}'.format(config_directory))
    logger.info('Model Directory = {}'.format(model_directory))

    try:
        command = MAVEN_COMMAND.format(model_directory)
        logger.debug(command)

        command_parts = command.split()
        proc = subprocess.Popen(
            command_parts,
            cwd=config_directory,
            stdout=subprocess.PIPE,
            shell=_shell_argument())

        # Calling proc.communicate() waits for the subprocess to complete.
        output, _ = proc.communicate()
        logger.info(output)
    except Exception:
        logger.error('Error while downloading jar files.', exc_info=True)
        raise

def recover_model_config():
    """
    Function to recover the model config if user accidentally deleted it
    """
    try:
        root_directory = get_root_directory()
        default_config, config_file_path = _parse_default_config(root_directory)
    
        with codecs.open(config_file_path, mode='w', encoding='utf-8') as file:
            default_config.write(file)
    except:
        logger.error("Error while recovering model config.", exc_info=True)


def get_root_directory():
    """Get the root config/model directory"""

    root_directory = os.path.expanduser(DEFAULT_CONFIG_ROOT_DIRECTORY)

    if not os.path.exists(root_directory):
        os.makedirs(root_directory)

    return root_directory


def get_model_path():
    """Returns the path of the JAR models."""

    root_directory = get_root_directory()
    default_config, _ = _parse_default_config(root_directory)
    version = default_config['model_download']['version']

    return os.path.join(root_directory, MODEL_FOLDER.format(version))


def main(args):
    """Default handler function"""

    logger.info("Starting download")
    try:
        _check_maven_installed()

        # Get default config along with args override.
        root_directory = get_root_directory()
        (config, config_file_path) = _parse_default_config(
            root_directory, args)
        version = config['model_download']['version']

        # Create/Update POM file.
        pom_file_path = os.path.join(root_directory, "pom.xml")
        logger.info(pom_file_path)
        _create_or_update_pom_file(pom_file_path, version)

        # Download model jars according to the version specified.
        jar_directory = os.path.join(root_directory,
                                     MODEL_FOLDER.format(version))
        _download_jars(jar_directory, root_directory, version)

        # Write the updated config file if download was successful.
        with codecs.open(config_file_path, mode='w', encoding='utf-8') as file:
            config.write(file)

        logger.info("Model download successful.")
    except Exception:
        logger.exception("Download failed.", exc_info=True)



import logging

from .view import *

logger = logging.getLogger(__name__)

class PredicateArgumentView(View):
    def __init__(self, view, tokens):
        super(PredicateArgumentView, self).__init__(view, tokens)

        self.predicates = []

        # The hypothesis is that all nodes with no incoming edges are predicates.
        for constituent in self.cons_list:
            if 'incoming_relations' not in constituent or len(constituent['incoming_relations']) == 0:
                self.predicates.append(constituent)

    def get_predicates(self):
        """
        Function to return a list of predicates which are constituents with no incoming relations

        @return: list of predicates
        """
        return self.predicates

    def get_arguments(self, predicate):
        """
        Function to return a list of outgoing relations given a predicate

        @param: predicate, the predicate to get outgoing relations from 
        @return: a list of relations, None if the predicate is invalid
        """
        if self._check_predicate(predicate):
            res = []
            for index in predicate['outgoing_relations']:
               res.append(self.get_relations(index))
            return res
        return None

    def get_predicate_properties(self, predicate):
        """
        Function to get the properties of given predicate

        @param: predicate, the predicate to get properties from
        @return: a dictionary object contains following keys: predicate, SenseNumber
        """
        if self._check_predicate(predicate):
            return predicate['properties']
        return None

    def _check_predicate(self, predicate):
        """
        Helper function to check if predicate is valid

        @param: predicate, the predicate to evaluate
        @return: True if the predicate is valid. False, otherwise
        """
        if predicate not in self.predicates:
            logger.error("Predicate " + predicate + " not found")
            return False
        else:
            return True



import json
import requests
import sys
import os
import logging

from abc import ABCMeta, abstractmethod
from backports.configparser import RawConfigParser

from .core.text_annotation import *
from .download import get_model_path
from . import pipeline_config

logger = logging.getLogger(__name__)

class PipelineBase:
    __metaclass__ = ABCMeta

    def __init__(self, file_name = None):
        """
        Constructor to load configuration of the pipeline base
        """
        if file_name is not None:
            self.config, self.models_downloaded = pipeline_config.get_user_config(file_name)
        else:
            self.config, self.models_downloaded = pipeline_config.get_current_config()


    def doc(self, text="Hello World", pretokenized=False):
        """
        Initialize text annotation of given text

        @param: text, the text to be processed
        @return: TextAnnotation instance of the text, None if text is empty
        """
        if not pretokenized:
            response = self.call_server(text, "TOKENS")

        else:
            response = self.call_server_pretokenized(text, "TOKENS")

        if response is not None:
            return TextAnnotation(response, self)
        else:
            return None


    @abstractmethod
    def call_server(text, views):
        """
        Funtion to get preprocess text annotation from server

        @param: text, the text to generate text annotation on
                views, the views to generate
        @return: raw text of the response from server
        """
        logger.error("This function should be overrided.")
        #raise NotImplementedError()
        return None


    @abstractmethod
    def call_server_pretokenized(pretokenized_text, views):
        """
        Funtion to get preprocess text annotation from server

        @param: pretokenized_text, list of list of tokens of pre-tokenized text
        @return: raw text of the response from server
        """
        logger.error("This function should be overrided.")
        #raise NotImplementedError()
        return None



    @abstractmethod
    def add_additional_views_to_TA(self, textannotation, views):
        """
        Funtion to add additional views to an existing TextAnnotation

        @param: textannotation, the python TA object
                views, the views to generate
        @return: raw text of the response from server -- jsonStr
        """
        logger.error("This function should be overrided.")
        #raise NotImplementedError()
        return None



import copy
from math import sqrt

from matplotlib.pyplot import ylim
from matplotlib.pyplot import xlim

import matplotlib.path as mpath
import matplotlib.patches as mpatches

Path = mpath.Path

from ccg_nlpy import TextAnnotation

import networkx as nx
import matplotlib.pyplot as plt


class TextVizualization():
    @classmethod
    def get_len(cls, t):
        return max(len(t['tokens']), t['label_len'], len(t['label']))

    @classmethod
    def do(cls, ta, view_name, break_sentences=True):
        constituents1 = ta.get_view(view_name).cons_list
        relations1 = ta.get_view(view_name).relation_array

        start_idx = 0
        figscale = 10
        height = 0.08
        scale = 0.039

        fig1 = plt.figure(figsize=(figscale * 2, figscale * 1.5), facecolor='white', edgecolor='black')
        ax1 = fig1.add_subplot(111, aspect='equal')

        sentence_end_position = [len(ta.tokens) + 1]
        if(break_sentences):
            sentence_end_position = ta.sentence_end_position

        tokens = []
        for i, t in enumerate(ta.get_view(view_name).tokens):
            token = {}
            token['start'] = i
            token['end'] = i + 1
            token['label'] = ""
            token['tokens'] = t
            token['label_len'] = 0
            tokens.append(token)

        for i, node in enumerate(constituents1):
            start_token = node["start"]
            end_token = node["end"]
            for token_idx in range(start_token, end_token):
                tokens[token_idx]["label_len"] = len(node["label"]) / (end_token - start_token)

        for sent_idx, end_idx in enumerate(sentence_end_position):
            y = -0.5 * sent_idx
            if (sent_idx > 0):
                start_idx = sentence_end_position[sent_idx - 1]

            offset = [0]
            for t_idx, t in enumerate(tokens):
                if (t_idx >= start_idx and t_idx < end_idx):
                    offset.append(offset[-1] + cls.get_len(t) + 1)
                    t["sentId"] = sent_idx

            for t_idx, node in enumerate(tokens):
                if (t_idx >= start_idx and t_idx < end_idx):
                    x = scale * offset[t_idx - start_idx]
                    width = scale * cls.get_len(node)
                    center_x = x + width / 2
                    center_y = y
                    node["x"] = center_x
                    plt.text(center_x, center_y - height, node['tokens'], ha="center", fontsize=figscale)

            for i, node in enumerate(constituents1):
                start = node["start"]
                end = node["end"]
                if (start >= start_idx and start < end_idx):
                    node["sentId"] = sent_idx
                    center_x = (tokens[start]["x"] + tokens[end - 1]["x"]) / 2
                    center_y = y
                    node["x"] = center_x
                    extra_whitespace = ""
                    if (len(node['tokens']) > len(node['label'])):
                        extra_whitespace = ' ' * int((len(node['tokens']) - len(node['label'])) / 2.0)
                    label = extra_whitespace + node['label'] + extra_whitespace
                    plt.text(center_x, center_y, label, ha="center", fontsize=figscale,
                             bbox=dict(alpha=0.1, facecolor='red', edgecolor='none', boxstyle='round')
                             )

            if(relations1 is not None):
                for r in relations1:
                    start_idx = r["srcConstituent"]
                    end_idx = r["targetConstituent"]
                    if ("sentId" in constituents1[start_idx] and constituents1[start_idx]["sentId"] == sent_idx):
                        if (sent_idx == 3):
                            print("Relation sentence 3 ")
                        start_x = constituents1[start_idx]["x"]
                        end_x = constituents1[end_idx]["x"]
                        middle_x = (start_x + end_x) / 2
                        middle_y = 0.8 * sqrt(abs(end_x - start_x))
                        pp1 = mpatches.PathPatch(
                            Path([(start_x, y + height), (middle_x, y + middle_y), (end_x, y + height)],
                                 [Path.MOVETO, Path.CURVE3, Path.CURVE3]),
                            fc="none", transform=ax1.transData)
                        ax1.add_patch(pp1)
                        # dy = -0.01
                        # offset = 0.05
                        # if(end_x > start_x):
                        #     dy = -dy
                        #     offset = -0.05
                        # pp2 = mpatches.FancyArrow(middle_x + offset, middle_y / 2, dy, 0.0, width=0.02, edgecolor="white", facecolor="black")
                        # ax1.add_patch(pp2)

        # for c in constituents1:
        #     print(c)
        #
        # for r in relations1:
        #     print(r)
        #
        # for t in tokens:
        #     print(t)

        ylim((-1, 1))
        xlim(-0.1, 7)
        plt.axis('off')
        plt.title('Generated with: CogComp-NLPy')
        plt.show()


def main():
    with open('../../tests/sample_text_annotation2.json', 'r') as myfile:
        data = myfile.read()
    ta = TextAnnotation(json_str=data)
    print(ta)
    print(ta.get_views)
    # TextVizualization.do(ta, "MENTION")
    TextVizualization.do(ta, "SRL_VERB", True)
    # TextVizualization.do(ta, "POS")

if __name__ == '__main__':
    main()



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: protobuf/TextAnnotation.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='protobuf/TextAnnotation.proto',
  package='textannotation',
  syntax='proto2',
  serialized_pb=_b('\n\x1dprotobuf/TextAnnotation.proto\x12\x0etextannotation\"\xe2\x02\n\x13TextAnnotationProto\x12\x10\n\x08\x63orpusId\x18\x01 \x02(\t\x12\n\n\x02id\x18\x02 \x02(\t\x12\x0c\n\x04text\x18\x03 \x02(\t\x12\x0e\n\x06tokens\x18\x04 \x03(\t\x12\x37\n\x0ctokenOffsets\x18\x05 \x03(\x0b\x32!.textannotation.TokenOffsetsProto\x12\x30\n\tsentences\x18\x06 \x02(\x0b\x32\x1d.textannotation.SentenceProto\x12(\n\x05views\x18\x07 \x03(\x0b\x32\x19.textannotation.ViewProto\x12G\n\nproperties\x18\x08 \x03(\x0b\x32\x33.textannotation.TextAnnotationProto.PropertiesEntry\x1a\x31\n\x0fPropertiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"Q\n\x11TokenOffsetsProto\x12\x0c\n\x04\x66orm\x18\x01 \x02(\t\x12\x17\n\x0fstartCharOffset\x18\x02 \x02(\x05\x12\x15\n\rendCharOffset\x18\x03 \x02(\x05\"R\n\rSentenceProto\x12\x11\n\tgenerator\x18\x01 \x02(\t\x12\x10\n\x05score\x18\x02 \x01(\x01:\x01\x30\x12\x1c\n\x14sentenceEndPositions\x18\x03 \x03(\x05\"N\n\tViewProto\x12\x10\n\x08viewName\x18\x01 \x02(\t\x12/\n\x08viewData\x18\x02 \x03(\x0b\x32\x1d.textannotation.ViewDataProto\"\xc2\x01\n\rViewDataProto\x12\x10\n\x08viewType\x18\x01 \x02(\t\x12\x10\n\x08viewName\x18\x02 \x02(\t\x12\x11\n\tgenerator\x18\x03 \x02(\t\x12\x10\n\x05score\x18\x04 \x01(\x01:\x01\x30\x12\x36\n\x0c\x63onstituents\x18\x05 \x03(\x0b\x32 .textannotation.ConstituentProto\x12\x30\n\trelations\x18\x06 \x03(\x0b\x32\x1d.textannotation.RelationProto\"\xca\x02\n\x10\x43onstituentProto\x12\r\n\x05label\x18\x01 \x02(\t\x12\x10\n\x05score\x18\x02 \x01(\x01:\x01\x30\x12\r\n\x05start\x18\x03 \x02(\x05\x12\x0b\n\x03\x65nd\x18\x04 \x02(\x05\x12\x44\n\nproperties\x18\x05 \x03(\x0b\x32\x30.textannotation.ConstituentProto.PropertiesEntry\x12J\n\rlabelScoreMap\x18\x06 \x03(\x0b\x32\x33.textannotation.ConstituentProto.LabelScoreMapEntry\x1a\x31\n\x0fPropertiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x34\n\x12LabelScoreMapEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x01:\x02\x38\x01\"\xdf\x02\n\rRelationProto\x12\x14\n\x0crelationName\x18\x01 \x02(\t\x12\x10\n\x05score\x18\x02 \x01(\x01:\x01\x30\x12\x16\n\x0esrcConstituent\x18\x03 \x02(\x05\x12\x19\n\x11targetConstituent\x18\x04 \x02(\x05\x12\x41\n\nproperties\x18\x05 \x03(\x0b\x32-.textannotation.RelationProto.PropertiesEntry\x12G\n\rlabelScoreMap\x18\x06 \x03(\x0b\x32\x30.textannotation.RelationProto.LabelScoreMapEntry\x1a\x31\n\x0fPropertiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x34\n\x12LabelScoreMapEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x01:\x02\x38\x01\x42G\n/edu.illinois.cs.cogcomp.core.utilities.protobufB\x12TextAnnotationImplP\x00')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_TEXTANNOTATIONPROTO_PROPERTIESENTRY = _descriptor.Descriptor(
  name='PropertiesEntry',
  full_name='textannotation.TextAnnotationProto.PropertiesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='textannotation.TextAnnotationProto.PropertiesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='textannotation.TextAnnotationProto.PropertiesEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=355,
  serialized_end=404,
)

_TEXTANNOTATIONPROTO = _descriptor.Descriptor(
  name='TextAnnotationProto',
  full_name='textannotation.TextAnnotationProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='corpusId', full_name='textannotation.TextAnnotationProto.corpusId', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='id', full_name='textannotation.TextAnnotationProto.id', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='text', full_name='textannotation.TextAnnotationProto.text', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='tokens', full_name='textannotation.TextAnnotationProto.tokens', index=3,
      number=4, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='tokenOffsets', full_name='textannotation.TextAnnotationProto.tokenOffsets', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='sentences', full_name='textannotation.TextAnnotationProto.sentences', index=5,
      number=6, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='views', full_name='textannotation.TextAnnotationProto.views', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='properties', full_name='textannotation.TextAnnotationProto.properties', index=7,
      number=8, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_TEXTANNOTATIONPROTO_PROPERTIESENTRY, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=50,
  serialized_end=404,
)


_TOKENOFFSETSPROTO = _descriptor.Descriptor(
  name='TokenOffsetsProto',
  full_name='textannotation.TokenOffsetsProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='form', full_name='textannotation.TokenOffsetsProto.form', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='startCharOffset', full_name='textannotation.TokenOffsetsProto.startCharOffset', index=1,
      number=2, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='endCharOffset', full_name='textannotation.TokenOffsetsProto.endCharOffset', index=2,
      number=3, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=406,
  serialized_end=487,
)


_SENTENCEPROTO = _descriptor.Descriptor(
  name='SentenceProto',
  full_name='textannotation.SentenceProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='generator', full_name='textannotation.SentenceProto.generator', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='score', full_name='textannotation.SentenceProto.score', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=True, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='sentenceEndPositions', full_name='textannotation.SentenceProto.sentenceEndPositions', index=2,
      number=3, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=489,
  serialized_end=571,
)


_VIEWPROTO = _descriptor.Descriptor(
  name='ViewProto',
  full_name='textannotation.ViewProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='viewName', full_name='textannotation.ViewProto.viewName', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='viewData', full_name='textannotation.ViewProto.viewData', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=573,
  serialized_end=651,
)


_VIEWDATAPROTO = _descriptor.Descriptor(
  name='ViewDataProto',
  full_name='textannotation.ViewDataProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='viewType', full_name='textannotation.ViewDataProto.viewType', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='viewName', full_name='textannotation.ViewDataProto.viewName', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='generator', full_name='textannotation.ViewDataProto.generator', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='score', full_name='textannotation.ViewDataProto.score', index=3,
      number=4, type=1, cpp_type=5, label=1,
      has_default_value=True, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='constituents', full_name='textannotation.ViewDataProto.constituents', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='relations', full_name='textannotation.ViewDataProto.relations', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=654,
  serialized_end=848,
)


_CONSTITUENTPROTO_PROPERTIESENTRY = _descriptor.Descriptor(
  name='PropertiesEntry',
  full_name='textannotation.ConstituentProto.PropertiesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='textannotation.ConstituentProto.PropertiesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='textannotation.ConstituentProto.PropertiesEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=355,
  serialized_end=404,
)

_CONSTITUENTPROTO_LABELSCOREMAPENTRY = _descriptor.Descriptor(
  name='LabelScoreMapEntry',
  full_name='textannotation.ConstituentProto.LabelScoreMapEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='textannotation.ConstituentProto.LabelScoreMapEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='textannotation.ConstituentProto.LabelScoreMapEntry.value', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1129,
  serialized_end=1181,
)

_CONSTITUENTPROTO = _descriptor.Descriptor(
  name='ConstituentProto',
  full_name='textannotation.ConstituentProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='label', full_name='textannotation.ConstituentProto.label', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='score', full_name='textannotation.ConstituentProto.score', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=True, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='start', full_name='textannotation.ConstituentProto.start', index=2,
      number=3, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='end', full_name='textannotation.ConstituentProto.end', index=3,
      number=4, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='properties', full_name='textannotation.ConstituentProto.properties', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='labelScoreMap', full_name='textannotation.ConstituentProto.labelScoreMap', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_CONSTITUENTPROTO_PROPERTIESENTRY, _CONSTITUENTPROTO_LABELSCOREMAPENTRY, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=851,
  serialized_end=1181,
)


_RELATIONPROTO_PROPERTIESENTRY = _descriptor.Descriptor(
  name='PropertiesEntry',
  full_name='textannotation.RelationProto.PropertiesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='textannotation.RelationProto.PropertiesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='textannotation.RelationProto.PropertiesEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=355,
  serialized_end=404,
)

_RELATIONPROTO_LABELSCOREMAPENTRY = _descriptor.Descriptor(
  name='LabelScoreMapEntry',
  full_name='textannotation.RelationProto.LabelScoreMapEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='textannotation.RelationProto.LabelScoreMapEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='textannotation.RelationProto.LabelScoreMapEntry.value', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1129,
  serialized_end=1181,
)

_RELATIONPROTO = _descriptor.Descriptor(
  name='RelationProto',
  full_name='textannotation.RelationProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='relationName', full_name='textannotation.RelationProto.relationName', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='score', full_name='textannotation.RelationProto.score', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=True, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='srcConstituent', full_name='textannotation.RelationProto.srcConstituent', index=2,
      number=3, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='targetConstituent', full_name='textannotation.RelationProto.targetConstituent', index=3,
      number=4, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='properties', full_name='textannotation.RelationProto.properties', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='labelScoreMap', full_name='textannotation.RelationProto.labelScoreMap', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_RELATIONPROTO_PROPERTIESENTRY, _RELATIONPROTO_LABELSCOREMAPENTRY, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1184,
  serialized_end=1535,
)

_TEXTANNOTATIONPROTO_PROPERTIESENTRY.containing_type = _TEXTANNOTATIONPROTO
_TEXTANNOTATIONPROTO.fields_by_name['tokenOffsets'].message_type = _TOKENOFFSETSPROTO
_TEXTANNOTATIONPROTO.fields_by_name['sentences'].message_type = _SENTENCEPROTO
_TEXTANNOTATIONPROTO.fields_by_name['views'].message_type = _VIEWPROTO
_TEXTANNOTATIONPROTO.fields_by_name['properties'].message_type = _TEXTANNOTATIONPROTO_PROPERTIESENTRY
_VIEWPROTO.fields_by_name['viewData'].message_type = _VIEWDATAPROTO
_VIEWDATAPROTO.fields_by_name['constituents'].message_type = _CONSTITUENTPROTO
_VIEWDATAPROTO.fields_by_name['relations'].message_type = _RELATIONPROTO
_CONSTITUENTPROTO_PROPERTIESENTRY.containing_type = _CONSTITUENTPROTO
_CONSTITUENTPROTO_LABELSCOREMAPENTRY.containing_type = _CONSTITUENTPROTO
_CONSTITUENTPROTO.fields_by_name['properties'].message_type = _CONSTITUENTPROTO_PROPERTIESENTRY
_CONSTITUENTPROTO.fields_by_name['labelScoreMap'].message_type = _CONSTITUENTPROTO_LABELSCOREMAPENTRY
_RELATIONPROTO_PROPERTIESENTRY.containing_type = _RELATIONPROTO
_RELATIONPROTO_LABELSCOREMAPENTRY.containing_type = _RELATIONPROTO
_RELATIONPROTO.fields_by_name['properties'].message_type = _RELATIONPROTO_PROPERTIESENTRY
_RELATIONPROTO.fields_by_name['labelScoreMap'].message_type = _RELATIONPROTO_LABELSCOREMAPENTRY
DESCRIPTOR.message_types_by_name['TextAnnotationProto'] = _TEXTANNOTATIONPROTO
DESCRIPTOR.message_types_by_name['TokenOffsetsProto'] = _TOKENOFFSETSPROTO
DESCRIPTOR.message_types_by_name['SentenceProto'] = _SENTENCEPROTO
DESCRIPTOR.message_types_by_name['ViewProto'] = _VIEWPROTO
DESCRIPTOR.message_types_by_name['ViewDataProto'] = _VIEWDATAPROTO
DESCRIPTOR.message_types_by_name['ConstituentProto'] = _CONSTITUENTPROTO
DESCRIPTOR.message_types_by_name['RelationProto'] = _RELATIONPROTO

TextAnnotationProto = _reflection.GeneratedProtocolMessageType('TextAnnotationProto', (_message.Message,), dict(

  PropertiesEntry = _reflection.GeneratedProtocolMessageType('PropertiesEntry', (_message.Message,), dict(
    DESCRIPTOR = _TEXTANNOTATIONPROTO_PROPERTIESENTRY,
    __module__ = 'protobuf.TextAnnotation_pb2'
    # @@protoc_insertion_point(class_scope:textannotation.TextAnnotationProto.PropertiesEntry)
    ))
  ,
  DESCRIPTOR = _TEXTANNOTATIONPROTO,
  __module__ = 'protobuf.TextAnnotation_pb2'
  # @@protoc_insertion_point(class_scope:textannotation.TextAnnotationProto)
  ))
_sym_db.RegisterMessage(TextAnnotationProto)
_sym_db.RegisterMessage(TextAnnotationProto.PropertiesEntry)

TokenOffsetsProto = _reflection.GeneratedProtocolMessageType('TokenOffsetsProto', (_message.Message,), dict(
  DESCRIPTOR = _TOKENOFFSETSPROTO,
  __module__ = 'protobuf.TextAnnotation_pb2'
  # @@protoc_insertion_point(class_scope:textannotation.TokenOffsetsProto)
  ))
_sym_db.RegisterMessage(TokenOffsetsProto)

SentenceProto = _reflection.GeneratedProtocolMessageType('SentenceProto', (_message.Message,), dict(
  DESCRIPTOR = _SENTENCEPROTO,
  __module__ = 'protobuf.TextAnnotation_pb2'
  # @@protoc_insertion_point(class_scope:textannotation.SentenceProto)
  ))
_sym_db.RegisterMessage(SentenceProto)

ViewProto = _reflection.GeneratedProtocolMessageType('ViewProto', (_message.Message,), dict(
  DESCRIPTOR = _VIEWPROTO,
  __module__ = 'protobuf.TextAnnotation_pb2'
  # @@protoc_insertion_point(class_scope:textannotation.ViewProto)
  ))
_sym_db.RegisterMessage(ViewProto)

ViewDataProto = _reflection.GeneratedProtocolMessageType('ViewDataProto', (_message.Message,), dict(
  DESCRIPTOR = _VIEWDATAPROTO,
  __module__ = 'protobuf.TextAnnotation_pb2'
  # @@protoc_insertion_point(class_scope:textannotation.ViewDataProto)
  ))
_sym_db.RegisterMessage(ViewDataProto)

ConstituentProto = _reflection.GeneratedProtocolMessageType('ConstituentProto', (_message.Message,), dict(

  PropertiesEntry = _reflection.GeneratedProtocolMessageType('PropertiesEntry', (_message.Message,), dict(
    DESCRIPTOR = _CONSTITUENTPROTO_PROPERTIESENTRY,
    __module__ = 'protobuf.TextAnnotation_pb2'
    # @@protoc_insertion_point(class_scope:textannotation.ConstituentProto.PropertiesEntry)
    ))
  ,

  LabelScoreMapEntry = _reflection.GeneratedProtocolMessageType('LabelScoreMapEntry', (_message.Message,), dict(
    DESCRIPTOR = _CONSTITUENTPROTO_LABELSCOREMAPENTRY,
    __module__ = 'protobuf.TextAnnotation_pb2'
    # @@protoc_insertion_point(class_scope:textannotation.ConstituentProto.LabelScoreMapEntry)
    ))
  ,
  DESCRIPTOR = _CONSTITUENTPROTO,
  __module__ = 'protobuf.TextAnnotation_pb2'
  # @@protoc_insertion_point(class_scope:textannotation.ConstituentProto)
  ))
_sym_db.RegisterMessage(ConstituentProto)
_sym_db.RegisterMessage(ConstituentProto.PropertiesEntry)
_sym_db.RegisterMessage(ConstituentProto.LabelScoreMapEntry)

RelationProto = _reflection.GeneratedProtocolMessageType('RelationProto', (_message.Message,), dict(

  PropertiesEntry = _reflection.GeneratedProtocolMessageType('PropertiesEntry', (_message.Message,), dict(
    DESCRIPTOR = _RELATIONPROTO_PROPERTIESENTRY,
    __module__ = 'protobuf.TextAnnotation_pb2'
    # @@protoc_insertion_point(class_scope:textannotation.RelationProto.PropertiesEntry)
    ))
  ,

  LabelScoreMapEntry = _reflection.GeneratedProtocolMessageType('LabelScoreMapEntry', (_message.Message,), dict(
    DESCRIPTOR = _RELATIONPROTO_LABELSCOREMAPENTRY,
    __module__ = 'protobuf.TextAnnotation_pb2'
    # @@protoc_insertion_point(class_scope:textannotation.RelationProto.LabelScoreMapEntry)
    ))
  ,
  DESCRIPTOR = _RELATIONPROTO,
  __module__ = 'protobuf.TextAnnotation_pb2'
  # @@protoc_insertion_point(class_scope:textannotation.RelationProto)
  ))
_sym_db.RegisterMessage(RelationProto)
_sym_db.RegisterMessage(RelationProto.PropertiesEntry)
_sym_db.RegisterMessage(RelationProto.LabelScoreMapEntry)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n/edu.illinois.cs.cogcomp.core.utilities.protobufB\022TextAnnotationImplP\000'))
_TEXTANNOTATIONPROTO_PROPERTIESENTRY.has_options = True
_TEXTANNOTATIONPROTO_PROPERTIESENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_CONSTITUENTPROTO_PROPERTIESENTRY.has_options = True
_CONSTITUENTPROTO_PROPERTIESENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_CONSTITUENTPROTO_LABELSCOREMAPENTRY.has_options = True
_CONSTITUENTPROTO_LABELSCOREMAPENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_RELATIONPROTO_PROPERTIESENTRY.has_options = True
_RELATIONPROTO_PROPERTIESENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_RELATIONPROTO_LABELSCOREMAPENTRY.has_options = True
_RELATIONPROTO_LABELSCOREMAPENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
# @@protoc_insertion_point(module_scope)



import json
import requests
import sys
import os
import logging

from backports.configparser import RawConfigParser

from .pipeline_base import *
from google.protobuf import json_format
from .protobuf import TextAnnotation_pb2
from .core.text_annotation import *
from .download import get_model_path
from . import pipeline_config
from . import utils

logger = logging.getLogger(__name__)

# set up JVM and
import jnius_config
try:
    jnius_config.add_options('-Xmx16G')
    jnius_config.add_classpath(get_model_path() + '/*')
except Exception as e:
    logger.warning(
        "Couldn't set JVM config; this might be because you're setting up the multiple instances of the local pipeline.")
    logger.warning(str(e))

# Don't import jnius before setting jnius_config
# Sec 4.5 : https://media.readthedocs.org/pdf/pyjnius/latest/pyjnius.pdf
from jnius import autoclass


class LocalPipeline(PipelineBase):
    def __init__(self):
        """
        Constructor to set up local pipeline

        @param: file_name, the file name of the custom config file
        """
        super(LocalPipeline, self).__init__()

        self.PipelineFactory = None
        self.pipeline = None
        self.ProtobufSerializer = None

        pipeline_config.log_current_config(self.config, False)

        # load java classes
        try:
            self.PipelineFactory = autoclass('edu.illinois.cs.cogcomp.pipeline.main.PipelineFactory')
            self.SerializationHelper = autoclass('edu.illinois.cs.cogcomp.core.utilities.SerializationHelper')
            self.IntPair = autoclass('edu.illinois.cs.cogcomp.core.datastructures.IntPair')
            self.TextAnnotation = autoclass('edu.illinois.cs.cogcomp.core.datastructures.textannotation.TextAnnotation')
            self.BasicTextAnnotationBuilder = autoclass('edu.illinois.cs.cogcomp.annotation.BasicTextAnnotationBuilder')
            self.BasicAnnotatorService = autoclass('edu.illinois.cs.cogcomp.annotation.BasicAnnotatorService')
            self.Tokenizer = autoclass('edu.illinois.cs.cogcomp.nlp.tokenizer.StatefulTokenizer')
            self.TextAnnotationBuilder = autoclass('edu.illinois.cs.cogcomp.nlp.utility.TokenizerTextAnnotationBuilder')
            # self.ProtobufSerializer = autoclass('edu.illinois.cs.cogcomp.core.utilities.protobuf.ProtobufSerializer')
            self.Boolean = autoclass('java.lang.Boolean')
            self.JString = autoclass('java.lang.String')
            self.JArrayList = autoclass('java.util.ArrayList')
        except Exception as e:
            logger.error('Fail to load models, please check if your Java version is up to date.')
            logger.error(str(e))
            return None
        self.pipeline = self.PipelineFactory.buildPipelineWithAllViews(self.Boolean(True))

        logger.info("pipeline has been set up")

    def call_server(self, text, views):
        """
        Funtion to get preprocess text annotation from local pipeline

        @param: text, the text to generate text annotation on
                views, the views to generate
        @return: raw text of the response from local pipeline
        """
        text = utils.strToBytes(text)
        view_list = views.split(',')
        text_annotation = self.pipeline.createBasicTextAnnotation(
            self.JString(""), self.JString(""), self.JString(text))
        for view in view_list:
            if (len(view.strip()) > 0):
                try:
                    self.pipeline.addView(text_annotation,
                                          self.JString(view.strip()))
                except Exception as e:
                    logger.error('Failed to add view ' + view.strip())
                    logger.error(str(e))

        jsonStr = self.SerializationHelper.serializeToJson(text_annotation)

        return jsonStr


    def add_additional_views_to_TA(self, textannotation, views):
        # Convert python textannotation to JsonStr -> Bytes
        jsonstrBytes = utils.strToBytes(json.dumps(textannotation.as_json))
        jsonStrJava = self.JString(jsonstrBytes)
        javaTA = self.SerializationHelper.deserializeFromJson(jsonStrJava)

        view_list = views.split(',')
        for view in view_list:
            view = view.strip()
            if (len(view) > 0):
                try:
                    self.pipeline.addView(javaTA, self.JString(view))
                except Exception as e:
                    logger.error('Failed to add view ' + view.strip())
                    logger.error(str(e))

        # Deserialize javaTA (woth additional views) to JsonStr
        jsonStr = self.SerializationHelper.serializeToJson(javaTA)

        return jsonStr


    def call_server_pretokenized(self, pretokenized_text, views):
        """
        Funtion to get preprocess text annotation from local pipeline

        @param: pretokenized_text, list of list of tokens of pre-tokenized text
        @return: raw text of the response from local pipeline
        """
        view_list = views.split(',')

        docAl = self.JArrayList()
        for sent in pretokenized_text:
            sentAL = self.JArrayList()
            for w in sent:
                w = utils.strToBytes(w)
                sentAL.add(self.JString(w))

            docAl.add(sentAL)

        text_annotation = self.BasicTextAnnotationBuilder.\
            createTextAnnotationFromListofListofTokens(docAl)

        for view in view_list:
            if (len(view.strip()) > 0):
                try:
                    self.pipeline.addView(text_annotation, self.JString(view.strip()))
                except Exception as e:
                    logger.error('Failed to add view ' + view.strip())
                    logger.error(str(e))

        jsonStr = self.SerializationHelper.serializeToJson(text_annotation)

        return jsonStr


    def doc_split_on_hyphens(self, text):
        tokenizer = self.Tokenizer()
        tab = self.TextAnnotationBuilder(tokenizer)

        text = utils.strToBytes(text)
        text_annotation = tab.createTextAnnotation(self.JString(""),
                                                   self.JString(""),
                                                   self.JString(text))



        jsonStr = self.SerializationHelper.serializeToJson(text_annotation)
        ta_python = TextAnnotation(json_str=jsonStr, pipeline_instance=self)

        return ta_python




    def call_server_with_sentences(self, sentences, views):
        """
        Funtion to get preprocess text annotation from local pipeline

        @param: sentences, the sentences to generate text annotation
                on views, the views to generate. sentences should be
                a list of lists of strings. Each string represents a token
                or word, each constituent list represents a sentence.
        @return: raw text of the response from local pipeline
        """
        view_list = views.split(',')
        tokens = [token for sent in sentences for token in sent]
        text = ' '.join(tokens)
        char_offsets = []
        count = 0
        char_offsets_ints = []
        for token in tokens:
            char_offsets.append(self.IntPair(count, count+len(token)-1+1))
            char_offsets_ints.append((count, count+len(token)-1+1))
            count += len(token)+1
        sentence_end_indices = []
        count = 0
        for sent in sentences:
            sentence_end_indices.append(count+len(sent)-1+1)
            count += len(sent)
        text = utils.strToBytes(text)
        text_annotation = self.TextAnnotation("", "", text, char_offsets, tokens, sentence_end_indices)
        for view in view_list:
            if (len(view.strip()) > 0):
                try:
                    self.pipeline.addView(text_annotation, self.JString(view.strip()))
                except Exception as e:
                    logger.error('Failed to add view ' + view.strip())
                    logger.error(str(e))

        json = self.SerializationHelper.serializeToJson(text_annotation)

        return json



import os
from setuptools import setup, find_packages

# Utility method to read the README.rst file.
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

import version
VERSION = version.__version__

CLASSIFIERS = [
    'Development Status :: 3 - Alpha', 'Intended Audience :: Science/Research',
    'Operating System :: Microsoft :: Windows', 'Operating System :: POSIX',
    'Operating System :: Unix', 'Operating System :: MacOS',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 2.6',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6', 'Topic :: Scientific/Engineering'
]

setup(
    name='ccg_nlpy',
    version=VERSION,
    description=("Light-weight NLP annotators"),
    long_description=read('README.md'),
    url='https://github.com/IllinoisCogComp/ccg_nlpy',
    author='Cognitive Computation Group',
    author_email='mssammon@illinois.edu',
    license='Research and Academic Use License',
    keywords="NLP, natural language processing",
    packages=find_packages(exclude=['tests.*', 'tests']),
    install_requires=['configparser', 'Cython', 'pyjnius', 'protobuf', 'requests', 'six'],
    package_data={'ccg_nlpy': ['config/*.cfg']},
    classifiers=CLASSIFIERS,
    setup_requires=['pytest-runner'],
    tests_require=['pytest', 'mock'],
    zip_safe=False)



import configparser
import codecs
import logging
import os
import sys

from . import download

logger = logging.getLogger(__name__)

CONFIG_FILENAME = 'config.cfg'
user_config_file = None

def get_current_config():
    """
    Function to get configuration for setting up pipeline.
    If the models have been downloaded, the function will (restore and) load configuration from '~/.ccg_nlpy/config.cfg.'
    Otherwise, it will load from 'pipeline.cfg' in the package

    @return: config, a ConfigParser instance with loaded configuration
             models_downloaded, True if models have been downloaded, False otherwise.

    """
    package_config_file = os.path.dirname(os.path.realpath(__file__)) + '/config/pipeline.cfg'
    config_file = package_config_file
    models_downloaded = os.path.exists(download.get_model_path())

    config = configparser.ConfigParser()

    # if model folder does not exist, then user hasn't download jars yet, use config file in the package
    # the config file in the package will use pipeline server instead of local pipeline
    download_dir = download.get_root_directory()
    default_config_file = os.path.join(download_dir, CONFIG_FILENAME)
    if models_downloaded:
        default_config_file = os.path.join(download_dir, CONFIG_FILENAME)
        if os.path.exists(default_config_file) is False:
            download.recover_model_config()
        config_file = default_config_file
    else:
        logger.warn('Models not found. To use pipeline locally, please refer the documentation for downloading models.')

    with codecs.open(config_file,mode='r',encoding='utf-8') as f:
        config.read_string(f.read())
    return config, models_downloaded

def get_user_config(file_name):
    """
    Function to get configuration for setting up pipeline from file that user provides.
    If the file does not exist, it will call function 'get_current_config()' and return its result

    @param: file_name, the file name of custom config file
    @return: config, a ConfigParser instance with loaded configuration
             models_downloaded, True if models have been downloaded, False otherwise
    """
    global user_config_file
    if file_name is not None and os.path.exists(file_name):
        models_downloaded = os.path.exists(download.get_model_path())
        config = configparser.ConfigParser()
        user_config_file = file_name
        with codecs.open(user_config_file,mode='r',encoding='utf-8') as f:
            config.read_string(f.read())
        return config, models_downloaded
    else:
        logger.warn('User config file not found, initializing pipeline with default config file.')
        return get_current_config()

def change_temporary_config(config, models_downloaded, use_server, server_api):
    """
    Function to change configuration temporarily. In other words, it only changes values in the ConfigParser provided as parameter.

    @param: config, the ConfigParser instance to be made changes on
            models_downloaded, Boolean to indicate if models have been downloaded
            enable_views, List of strings to indicate views to be enabled
            disable_views, List of strings to indicate views to be disabled (enable_views has higher priority)
            use_server, Boolean to indicate if using remote server
            server_api, the address of the server, including port number
    @return: List of names of enabled view if use_server is False, None otherwise
    """
    if server_api is not None:
        config['remote_pipeline_setting']['api'] = server_api

    log_current_config(config, use_server)

def set_current_config(config):
    """
    Function to write the current configuration to file if the configuration was loaded from custom config file

    @param: config, the ConfigParser instance to be written to file
    """
    if user_config_file is None:
        logger.error('Could not overwrite config file if user has not previous provide one.')
    else:
        with codecs.open(user_config_file, mode='w', encoding='utf-8') as file:
            config.write(file)
        logger.info('Config file has been updated.')


def log_current_config(config, use_server):
    """
    Function to log current configuration

    @param: config, the ConfigParser instance to be logged
            use_server, Boolean to indicate if using remote server
    """
    if use_server:
        logger.info('Using pipeline web server with API: {0}'.format(config['remote_pipeline_setting']['api']))
    else:
        logger.info('Using local pipeline')



from __future__ import print_function

import argparse

from .download import main as download_main

parser = argparse.ArgumentParser(prog='ccg_nlpy')
subparsers = parser.add_subparsers(help='commands')

# Create the parser for download command
parser_download = subparsers.add_parser('download', help='download help')
parser_download.add_argument(
    "--version", help="CogComp model version", required=False)
parser_download.set_defaults(func=download_main)

if __name__ == "__main__":
    import logging

    # Set the logging level for this mode.
    logging.basicConfig(level=logging.DEBUG)

    args = parser.parse_args()

    try:
        args.func(args)
    except AttributeError:
        parser.print_help()



import json
import logging

from .view import *
from .predicate_argument_view import *

logger = logging.getLogger(__name__)

class TextAnnotation(object):
    '''
        This class is was designed to be a python version of the TextAnnotation class.
        While most of the queries on view are done with View by querying on
        The JSON format of the Text Annotation (since the JSON string obtained from SerializationHelper
         is the same as the JSON format returned from web server)

    '''

    def __init__(self, json_str=None, pipeline_instance=None):
        result_json = json.loads(json_str)

        self.pipeline = pipeline_instance
        self.corpusId = result_json["corpusId"]
        self.id = result_json["id"]
        # self.text = result_json["text"].strip()
        self.text = result_json["text"] # always take text verbatim!
        if len(self.text) <= 0:
            logger.warn("Creating empty TextAnnotation.")
            self.empty = True
        else:
            self.empty = False
        self.tokens = result_json["tokens"]
        self.score = result_json["sentences"]["score"]
        self.sentences = result_json["sentences"]
        self.sentence_end_position = self.sentences["sentenceEndPositions"]
        if "tokenOffsets" in result_json:
            # Converting from list of dict to list of tuples
            # See function description for details
            tokenOffsets = self._convertTokenOffsetsDictToTupledList(
                result_json["tokenOffsets"])
            self.char_offsets = tokenOffsets
        else:
            self.char_offsets = self._extract_char_offset(self.text, self.tokens)
        self.view_dictionary = {}
        for view in result_json["views"]:
            self.view_dictionary[view["viewName"]] = self._view_builder(view)

    def _view_builder(self, view):
        full_type = view["viewData"][0]["viewType"]
        split_by_period = full_type.split(".")
        view_type = split_by_period[len(split_by_period) - 1]
        if view_type == 'PredicateArgumentView':
            return PredicateArgumentView(view, self.tokens)
        else:
            return View(view, self.tokens)

    def _extract_char_offset(self, sentence, tokens):
        """
        a function to extract char offsets given tokens and raw string
        Originally implemented in
        https://github.com/CogComp/cogcomp-nlp/blob/12cf80eabc92fe69ff7a53709f6d6e91fb00687d/core-utilities/src/main/java/edu/illinois/cs/cogcomp/core/utilities/TokenUtils.java#L27
        """
        offsets = []

        tokenId = 0
        characterId = 0

        tokenCharacterStart = 0
        tokenLength = 0

        while (characterId < len(sentence)
                 and sentence[characterId].isspace()):
            characterId = characterId + 1

        while (characterId < len(sentence)):
            if (tokenLength == len(tokens[tokenId])):
                offsets.append((tokenCharacterStart, characterId))

                while (characterId < len(sentence) and sentence[characterId].isspace()):
                    characterId = characterId + 1

                tokenCharacterStart = characterId
                tokenLength = 0
                tokenId = tokenId + 1

            else:
                assert sentence[characterId] == tokens[tokenId][tokenLength], sentence[characterId] + " expected, found " + tokens[tokenId][tokenLength] + " instead in sentence: " + sentence;
                tokenLength = tokenLength + 1
                characterId = characterId + 1

        if (characterId == len(sentence) and len(offsets) == len(tokens) - 1):
            offsets.append((tokenCharacterStart, len(sentence)))

        assert len(offsets) == len(tokens), offsets

        return offsets

    # Functions to manipulate the views on text annotation

    @property
    def get_pos(self):
        """
        Wrapper on getting part-of-speech tagger from given text annotation
        @param: text_annotation, TextAnnotation instance to get POS tagger from
        @return: View instance of POS tagger
        """
        return self.get_view("POS")

    @property
    def get_dependency_parse(self):
        """
        Wrapper on getting the dependency from given text annotation
        @param: text_annotation, TextAnnotation instance to get dependency view from
        @return: View instance of the dependency
        """
        return self.get_view("DEPENDENCY")

    @property
    def get_stanford_dependency_parse(self):
        """
        Wrapper on getting the dependency from given text annotation
        @param: text_annotation, TextAnnotation instance to get dependency view from
        @return: View instance of the dependency
        """
        return self.get_view("DEPENDENCY_STANFORD")

    @property
    def get_ner_conll(self):
        """
        Wrapper on getting the NER_CONLL view from given text annotation
        @param: text_annotation TextAnnotation instance to get NER_CONLL view from.
        @return: View Instance of the NER_CONLL view.
        """
        return self.get_view("NER_CONLL")

    @property
    def get_ner_ontonotes(self):
        """
        Wrapper on getting the NER_ONTONOTES view from given text annotation
        @param: text_annotation TextAnnotation instance to get NER_ONTONOTES view from.
        @return: View Instance of the NER_ONTONOTES view.
        """
        return self.get_view("NER_ONTONOTES")

    @property
    def get_stanford_parse(self):
        """
        Wrapper on getting the PARSE_STANFORD view from given text annotation
        @param: text_annotation TextAnnotation instance to get PARSE_STANFORD view from.
        @return: View Instance of the PARSE_STANFORD view.
        """
        return self.get_view("PARSE_STANFORD")

    @property
    def get_srl_verb(self):
        """
        Wrapper on getting the SRL_VERB view from given text annotation
        @param: text_annotation TextAnnotation instance to get SRL_VERB view from.
        @return: View Instance of the SRL_VERB view.
        """
        return self.get_view("SRL_VERB")

    @property
    def get_srl_nom(self):
        """
        Wrapper on getting the SRL_NOM view from given text annotation
        @param: text_annotation TextAnnotation instance to get SRL_NOM view from.
        @return: View Instance of the SRL_NOM view.
        """
        return self.get_view("SRL_NOM")

    @property
    def get_srl_prep(self):
        """
        Wrapper on getting the SRL_PREP view from given text annotation
        @param: text_annotation TextAnnotation instance to get SRL_PREP view from.
        @return: View Instance of the SRL_PREP view.
        """
        return self.get_view("SRL_PREP")

    @property
    def get_srl_comma(self):
        """
        Wrapper on getting the SRL_COMMA view from given text annotation
        @param: text_annotation TextAnnotation instance to get SRL_COMMA view from.
        @return: View Instance of the SRL_COMMA view.
        """
        return self.get_view("SRL_COMMA")

    @property
    def get_quantities(self):
        """
        Wrapper on getting the QUANTITIES view from given text annotation
        @param: text_annotation TextAnnotation instance to get QUANTITIES view from.
        @return: View Instance of the QUANTITIES view.
        """
        return self.get_view("QUANTITIES")

    @property
    def get_shallow_parse(self):
        """
        Wrapper on getting the SHALLOW_PARSE view from given text annotation
        @param: text_annotation TextAnnotation instance to get SHALLOW_PARSE view from.
        @return: View Instance of the SHALLOW_PARSE view.
        """
        return self.get_view("SHALLOW_PARSE")

    @property
    def get_lemma(self):
        """
        Wrapper on getting the LEMMA view from given text annotation
        @param: text_annotation TextAnnotation instance to get LEMMA view from.
        @return: View Instance of the LEMMA view.
        """
        return self.get_view("LEMMA")

    def add_view(self, view_name, response):
        result_json = json.loads(response)

        # check if the view is retrieve correctly
        # iterate through all the views presented in response, add the view that hasn't been store,
        # return the view requested
        requested_view = None
        view_constituents = []
        for view in result_json["views"]:
            name = view["viewName"]
            view_constituents.append(name)
            if name not in self.view_dictionary:
                self.view_dictionary[name] = self._view_builder(view)

            if name == view_name:
                requested_view = self.view_dictionary[name]

        if requested_view is None:
            # "token" view will always be included
            if len(view_constituents) <= 1:
                logger.error("Invalid view name, please check.")
            else:
                logger.info("The view is the collection of the following views: {0}".format(view_constituents))
                self.view_dictionary[name] = view_constituents
        return requested_view

    def get_view(self, view_name):
        if self.empty:
            logger.error("No valid views on empty TextAnnotation.")
            return None
        if view_name not in self.view_dictionary:
            additional_response = \
                self.pipeline.add_additional_views_to_TA(self, view_name)
            self.add_view(view_name, additional_response)

        if type(self.view_dictionary[view_name]) != type([]):
            return self.view_dictionary[view_name]
        else:
            logger.info("The view is the collection of the following views: {0}".format(self.view_dictionary[view_name]))
            return None

    @property
    def get_views(self):
        return list(self.view_dictionary.keys())

    # Functions to get general information about the text annotation

    @property
    def get_text(self):
        return self.text

    @property
    def get_tokens(self):
        return self.tokens

    @property
    def get_token_char_offsets(self):
        """
        Returns char offsets for tokens.
        For example, if the input string is:
        "Hello, how are you. I am doing fine"
        which is tokenized into this list:  [u'Hello', u',', u'how', u'are', u'you', u'.', u'I', u'am', u'doing', u'fine']
        calling this function should return the following list of pairs:

        [(0, 5), (5, 6), (7, 10), (11, 14), (15, 18), (18, 19), (20, 21), (22, 24), (25, 30), (31, 35)]
        """
        return self.char_offsets

    @property
    def get_score(self):
        return self.score

    @property
    def get_sentence_end_token_indices(self):
        return self.sentence_end_position

    @property
    def as_json(self):
        tokenOffsetsDicts = self._convertTokenOffsetsToDict()
        output = {
            "corpusId": self.corpusId,
            "id": self.id,
            "text": self.text,
            "tokens": self.tokens,
            "tokenOffsets": tokenOffsetsDicts,
            "sentences": self.sentences,
            "views": [v.as_json for v in self.view_dictionary.values()]

        }
        return output

    def _convertTokenOffsetsDictToTupledList(self, tokenOffsetsDicts):
        if not isinstance(tokenOffsetsDicts[0], dict):
            # Looks like the elements are lists, hence no conversion
            return tokenOffsetsDicts

        '''
            tokenOffsetsDicts: list of dicts with
            "form", "startCharOffset", and  "endCharOffset" keys
            We only need "startCharOffset", and  "endCharOffset" in a tuple
        '''
        tokenOffsets = []
        for offsetDict in tokenOffsetsDicts:
            tokenOffsets.append((offsetDict["startCharOffset"],
                                 offsetDict["endCharOffset"]))

        return tokenOffsets

    def _convertTokenOffsetsToDict(self):
        tokenOffsetsDicts = []
        for offsetPair, token in zip(self.char_offsets, self.tokens):
            tokenOffsetsDicts.append(
                {
                    "form": token,
                    "startCharOffset": offsetPair[0],
                    "endCharOffset": offsetPair[1]
                }
            )
        return tokenOffsetsDicts



import json
import requests
import sys
import os
import logging

from backports.configparser import RawConfigParser

from .pipeline_base import *
from google.protobuf import json_format
from .protobuf import TextAnnotation_pb2
from .core.text_annotation import *
from .download import get_model_path
from . import pipeline_config
from . import utils

WEB_SERVER_SUFFIX = '/annotate'

WEB_SERVER_SUFFIX_ADDVIEWS = '/addviews'


logger = logging.getLogger(__name__)

class RemotePipeline(PipelineBase):
    def __init__(self, server_api=None, file_name=None):
        """
        Constructor to set up remote pipeline

        @param: server_api, the address and port of the server (ex. http://fancyUrlName.com:8080)
                file_name, the file name of custom config file
        """
        super(RemotePipeline,self).__init__(file_name)

        # reroute to new API if user provides one
        pipeline_config.change_temporary_config(self.config, self.models_downloaded, True, server_api)
        self.url = self.config['remote_pipeline_setting']['api']

        logger.info("pipeline has been set up")

    def call_server(self, text, views):
        """
        Funtion to get preprocess text annotation from server

        @param: text, the text to generate text annotation on
                views, the views to generate
        @return: raw text of the response from server
        """
        response = None
        try:
            data = {'text': text, 'views': views}
            response = requests.post(self.url+WEB_SERVER_SUFFIX, data)
        except:
            logger.error("Fail to connect to server.")
            raise

        try:
            if response.status_code == 200:
                return response.text
            elif response.status_code == 429:
                logger.error("You reached maximum query limit with default remote server (100 queries/day)")
                raise Exception("You reached maximum query limit with default remote server (100 queries/day)")
            else:
                logger.warn("Unexpected status code {}, please open an issue on GitHub for further investigation.".format(response.status_code))
                return None
        except:
            raise


    def call_server_pretokenized(self, pretokenized_text, views):
        """
            Text Annotation for pre-tokenized text is not implemented for
            remote pipeline. Use local pipeline instead.
        """

        raise NotImplemented


    def add_additional_views_to_TA(self, textannotation, views):
        response = None
        try:
            jsonStrTA = json.dumps(textannotation.as_json)
            data = {'jsonstr': jsonStrTA, 'views': views}
            response = requests.post(self.url+WEB_SERVER_SUFFIX_ADDVIEWS, data)
        except:
            logger.error("Fail to connect to server.")
            raise

        try:
            if response.status_code == 200:
                return response.text
            elif response.status_code == 429:
                logger.error("You reached maximum query limit with default remote server (100 queries/day)")
                raise Exception("You reached maximum query limit with default remote server (100 queries/day)")
            else:
                logger.warn("Unexpected status code {}, please open an issue on GitHub for further investigation.".format(response.status_code))
                return None
        except:
            raise
